==> diffsptk-2.0.1.tar.gz: diffsptk-2.0.1/diffsptk/modules/lar2par.py <==
# ------------------------------------------------------------------------ #
# Copyright 2022 SPTK Working Group                                        #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http://www.apache.org/licenses/LICENSE-2.0                           #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
# ------------------------------------------------------------------------ #

import torch
from torch import nn

from ..misc.utils import check_size


class LogAreaRatioToParcorCoefficients(nn.Module):
    """See `this page <https://sp-nitech.github.io/sptk/latest/main/lar2par.html>`_
    for details.

    Parameters
    ----------
    par_order : int >= 0
        Order of PARCOR, :math:`M`.

    """

    def __init__(self, par_order):
        super().__init__()

        assert 0 <= par_order

        self.par_order = par_order

    def forward(self, g):
        """Convert LAR to PARCOR.

        Parameters
        ----------
        g : Tensor [shape=(..., M+1)]
            Log area ratio.

        Returns
        -------
        out : Tensor [shape=(..., M+1)]
            PARCOR coefficients.

        Examples
        --------
        >>> g = diffsptk.ramp(1, 4) * 0.1
        >>> lar2par = diffsptk.LogAreaRatioToParcorCoefficients(3)
        >>> k = lar2par(g)
        >>> k
        tensor([0.1000, 0.0997, 0.1489, 0.1974])

        """
        check_size(g.size(-1), self.par_order + 1, "dimension of parcor")
        return self._forward(g)

    @staticmethod
    def _forward(g):
        K, g = torch.split(g, [1, g.size(-1) - 1], dim=-1)
        k = torch.cat((K, torch.tanh(0.5 * g)), dim=-1)
        return k

    _func = _forward

==> diffsptk-3.0.0.tar.gz: diffsptk-3.0.0/diffsptk/modules/lar2par.py <==
# ------------------------------------------------------------------------ #
# Copyright 2022 SPTK Working Group                                        #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License");          #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http://www.apache.org/licenses/LICENSE-2.0                           #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
# ------------------------------------------------------------------------ #

import torch

from ..utils.private import check_size
from ..utils.private import get_values
from .base import BaseFunctionalModule


class LogAreaRatioToParcorCoefficients(BaseFunctionalModule):
    """See `this page <https://sp-nitech.github.io/sptk/latest/main/lar2par.html>`_
    for details.

    Parameters
    ----------
    par_order : int >= 0
        The order of the PARCOR coefficients, :math:`M`.

    """

    def __init__(self, par_order):
        super().__init__()

        self.in_dim = par_order + 1

        self.values = self._precompute(*get_values(locals()))

    def forward(self, g):
        """Convert LAR to PARCOR.

        Parameters
        ----------
        g : Tensor [shape=(..., M+1)]
            The log area ratio.

        Returns
        -------
        out : Tensor [shape=(..., M+1)]
            The PARCOR coefficients.

        Examples
        --------
        >>> g = diffsptk.ramp(1, 4) * 0.1
        >>> lar2par = diffsptk.LogAreaRatioToParcorCoefficients(3)
        >>> k = lar2par(g)
        >>> k
        tensor([0.1000, 0.0997, 0.1489, 0.1974])

        """
        check_size(g.size(-1), self.in_dim, "dimension of parcor")
        return self._forward(g, *self.values)

    @staticmethod
    def _func(x, *args, **kwargs):
        values = LogAreaRatioToParcorCoefficients._precompute(
            x.size(-1) - 1, *args, **kwargs
        )
        return LogAreaRatioToParcorCoefficients._forward(x, *values)

    @staticmethod
    def _takes_input_size():
        return True

    @staticmethod
    def _check(par_order):
        if par_order < 0:
            raise ValueError("par_order must be non-negative.")

    @staticmethod
    def _precompute(par_order):
        LogAreaRatioToParcorCoefficients._check(par_order)
        return (0.5,)

    @staticmethod
    def _forward(g, c):
        K, g = torch.split(g, [1, g.size(-1) - 1], dim=-1)
        k = torch.cat((K, torch.tanh(c * g)), dim=-1)
        return k
